#' RainFARM Stochastic Precipitation Downscaling Weights
#'
#' @author Jost von Hardenberg, ISAC-CNR, \email{j.vonhardenberg@isac.cnr.it}
#'
#' @description Compute orographic weights from a fine-scale precipitation climatology file.
#'
#' Reference: Terzago, S., Palazzi, E., & Von Hardenberg, J. (2018). Stochastic downscaling of precipitation in complex orography: A simple method to reproduce a realistic fine-scale climatology. Natural Hazards and Earth System Sciences, 18(11), 2825–2840. http://doi.org/10.5194/nhess-18-2825-2018 .
#'
#' Please notice: The rfweights() function uses julia_setup from the JuliaCall library, 
#' which takes a long time only the first time it is called on a machine
#' 
#' @param orofile filename of a fine-scale precipitation climatology 
#' @param nf refinement factor for downscaling (the output resolution is increased by this factor)
#' @param lon vector or array of longitudes (can be left out if \code{reffile} is used)
#' @param lat vector or array of latitudes (can be left out if \code{reffile} is used)
#' @param reffile filename of reference file (e.g. the file to downscale). Not needed if \code{lon} and \code{lat} are used
#' @param varname optional name of variable to be read from reffile
#' @param weightsfn optional filename to save the weights in netcdf format
#' @param fsmooth logical to use smooth conservation (default) or large-scale box-average conservation
#' @return The matrix containing the weights with dimensions (lon, lat).
#' @import ncdf4
#' @import JuliaCall
#' @examples
#'  
#' # Create weights to be used with CST_RainFARM() or RainFARM()
#' # Specify a weightsfn filename if you wish to save the weights
#' # Using an external fine-scale climatology file.
#' 
#' # Specify lon and lat of the input
#' lon_mat <- seq(10,13.5,0.5) # could also be a 2d matrix
#' lat_mat <- seq(40,43.5,0.5)
#' nf <- 8
#' ww <- rfweights("./worldclim.nc", nf, lon=lon_mat, lat=lat_mat, weightsfn="", fsmooth = TRUE)
#'
#' # or use a reference file (e.g. the input file itself) to specify the coordinates
#' ww <- rfweights("./worldclim.nc", nf, reffile="./input.nc", weightsfn="", fsmooth = TRUE)
#' @export

rfweights <- function(orofile, nf, lon=NULL, lat=NULL, reffile="",
                      weightsfn="", varname="", fsmooth=TRUE) {

#  library(JuliaCall)
#  library(ncdf4)
  julia_setup(useRCall = FALSE)
  julia_library("RainFARM")
  if ( (reffile == "") & ( (typeof(lon) == "NULL") | (typeof(lat) == "NULL"))) {
     stop("Neither a reference filename nor lon and lat specified!\n")
  }
  if (reffile == "") {
     xvar <- ncdim_def("lon", "degrees_east", lon, longname = "longitude")
     yvar <- ncdim_def("lat", "degrees_north", lat, longname = "latitude")
     tvar <- ncdim_def("time", "days since 01-01-2000", 0., longname = "time",
                       unlim = T)
     my_ncdf <- ncvar_def("grid", "m", list(xvar, yvar, tvar), -999,
                          longname = "grid", prec = "single")
     reffile <- tempfile()
     ncfile <- nc_create(reffile, my_ncdf, force_v4 = FALSE)
     nc_close(ncfile)
  }

  ww <- julia_call("rfweights", orofile, reffile, as.integer(nf),
                   weightsfn = weightsfn, varname = varname, fsmooth = fsmooth)
  
  unlink(reffile)
  return(ww)
}
